@@ -1,5 +1,5 @@
 package Bio::ToolBox::Data::Stream;
-our $VERSION = 1.26;
+our $VERSION = '1.30';
 
 =head1 NAME
 
@@ -11,13 +11,19 @@ Bio::ToolBox::Data::Stream - Read, Write, and Manipulate Data File Line by Line
   
   ### Open a pre-existing file
   my $Stream = Bio::ToolBox::Data->new(
-        file    => 'regions.bed',
+        in      => 'regions.bed',
         stream  => 1,
   );
   
   # or directly
   my $Stream = Bio::ToolBox::Data::Stream->new(
-        file    => 'regions.bed',
+        in      => 'regions.bed',
+  );
+  
+  ### Open a new file for writing
+  my $Stream = Bio::ToolBox::Data::Stream->new(
+        out     => 'output.txt',
+        columns => [qw(chromosome start stop name)],
   );
   
   
@@ -93,7 +99,7 @@ may also be opened using the L<Bio::ToolBox::Data> new function.
 	
 	my $Stream = Bio::ToolBox::Data->new(
 	   stream       => 1,
-	   file         => $filename,
+	   in           => $filename,
 	);
 
 Options to the new function are listed below. Streams are inherently either 
@@ -101,23 +107,51 @@ read or write mode, determined by the mode given through the options.
 
 =over 4
 
-=item file =E<gt> $filename
+=item in =E<gt> $filename
 
-Provide the path and name of the file to open. File types are recognized by 
-the extension, and compressed files (.gz) are supported. File types supported 
-include all those listed in L<Bio::ToolBox::file_helper>. Files are 
-checked for existence. Existing files are assumed to be read, and non-existent 
-files are assumed to be written, unless otherwise specified by the mode 
-option. This option is required.
+Provide the path and name of the file to open for reading. File types are 
+recognized by the extension, and compressed files (.gz) are supported. File 
+types supported include all those listed in L<Bio::ToolBox::file_helper>. 
 
-=item overwrite =E<gt> boolean
+=item out =E<gt> $filename
 
-If a file exists and you wish to overwrite, pass this option with a true value.
+Provide the path and name of the file to open for writing. No check is made 
+for pre-existing files; if it exists it will be overwritten! A new data 
+object is prepared, therefore column names must be provided. 
 
 =item columns =E<gt> [qw(Column1 Column2 ...)]
 
 When a new file is written, provide the names of the columns as an 
-anonymous array. 
+anonymous array. If no columns are provided, then a completely empty 
+data structure is made. Columns must be added with the add_column() 
+method below.
+
+=item gff =E<gt> $gff_version
+
+When writing a GFF file, provide a GFF version. When this is given, the 
+nine standard column names and metadata are automatically provided based 
+on the file format specification. Note that the column names are not 
+actually written in the file, but are maintained for internal use. 
+Acceptable versions include 1, 2, 2.5 (GTF), and 3 (GFF3).
+
+=item bed =E<gt> $number_of_bed_columns
+
+When writing a BED file, provide the number of bed columns that the file 
+will have. When this is given, the standard column names and metadata 
+will be automatically provided based on the standard file format 
+specification. Note that column names are not actually written to the file, 
+but are maintained for internal use. Acceptable values are integers from 
+3 to 12. 
+
+=item ucsc =E<gt> $number_of_columns
+
+When writing a UCSC-style file format, provide the number of bed columns 
+that the file will have. When this is given, the standard column names and 
+metadata will be automatically provided based on the file format specification. 
+Note that column names are not actually written to the file, but are maintained 
+for internal use. Acceptable values include 10 (refFlat without gene names), 
+11 (refFlat with gene names), 12 (knownGene gene prediction table), and 15 
+(an extended gene prediction or genePredExt table).
 
 =back
 
@@ -195,12 +229,7 @@ such. A value of 0 means they are not formatted as such.
 
 =over 4
 
-=item filename($text)
-
-Returns or sets the filename for the Data structure. If you set 
-a new filename, the path, basename, and extension are 
-automatically derived for you. If a path was not provided, 
-the current working directory is assumed. 
+=item filename
 
 =item path
 
@@ -208,9 +237,14 @@ the current working directory is assumed.
 
 =item extension
 
-Returns the full path, basename, and extension of the filename. 
-Concatenating these three values will reconstitute the 
-original filename.
+Returns the filename, full path, basename, and extension of 
+the filename. Concatenating the last three values will reconstitute 
+the first original filename.
+
+=item add_file_metadata($filename)
+
+Add filename metadata. This will automatically parse the path, 
+basename, and recognized extension from the passed filename.
 
 =back
 
@@ -444,125 +478,147 @@ the file handle. Use with caution.
 
 =cut
 
-
 use strict;
 use Carp qw(carp cluck croak confess);
-
-use Bio::ToolBox::data_helper qw(
-	generate_data_structure
-	find_column_index
-);
-use Bio::ToolBox::file_helper qw(
-	open_data_file
-	write_data_file
-	open_to_write_fh
-	parse_filename
-	process_data_line
-	check_file
-);
-use Bio::ToolBox::Data::common;
+use base 'Bio::ToolBox::Data::core';
 use Bio::ToolBox::Data::Feature;
 
 1;
 
 
-
 #### Initialize ####
 
 sub new {
 	my $class = shift;
 	my %args  = @_;
 	
-	$args{features} ||= $args{feature} || 'feature';
-	$args{filename} ||= $args{file} || undef;
-	$args{overwrite} ||= 0;
-	
-	unless (defined $args{filename}) {
-		carp "a filename must be provided!";
+	# file arguments
+	$args{in}  ||= undef;
+	$args{out} ||= undef;
+	unless ($args{in} or $args{out}) {
+		if (exists $args{filename} or exists $args{file}) {
+			warn "You are using an outdated API! Please update your code!\n";
+		}
+		cluck "a filename must be specified with 'in' or 'out' argument keys!\n";
 		return;
 	}
+	if (defined $args{in} and defined $args{out}) {
+		cluck "cannot define both 'in' and 'out' arguments!\n";
+		return;
+	} 
 	
-	# prepare
-	my $data;
-	my $fh;
-	my $mode;
+	# prepare object
+	my $self = $class->SUPER::new();
 	
-	# check if the file exists
-	my $filename = check_file($args{filename});
-	
-	# default behavior is to open an existing file for reading
-	if ($filename and not $args{overwrite}) {
-		($fh, $data) = open_data_file($filename);
-		unless ($data) {
-			croak "unable to read file $filename!";
-		}
-		$mode = 0; # read mode
+	# open an existing file for reading
+	if ($args{in}) {
 		
-		# add the column headers
-		push @{ $data->{data_table} }, $data->{'column_names'};
-		delete $data->{'column_names'}; # we no longer need this
-		
-		# look for potential start columns to convert from 0-based
-		my @starts;
-		if ($data->{'ucsc'} or $data->{'bed'}) {
-			foreach my $name (qw(start txStart cdsStart peak)) {
-				my $c = find_column_index($data, $name);
-				next unless defined $c;
-				next if (exists $data->{$c}{'base'} and $data->{$c}{'base'} == 1);
-				push @starts, $c;
-			}
-		}
-		$data->{column_starts} = \@starts;
-		
-		# potential strand column that may need to be converted
-		my $strand_i = find_column_index($data, '^strand$');
-		if (defined $strand_i) {
-			$data->{strand_check} = $strand_i;
-		}
-	}
-	
-	# alternate behavior if file does not exist is to create an empty stream
-	# and prepare it for writing
-	else {
-		my @datasets;
-		if (exists $args{datasets}) {
-			@datasets = @{ $args{datasets} };
-		}
-		elsif (exists $args{columns}) {
-			@datasets = @{ $args{columns} };
-		}
-		unless (@datasets) {
-			carp "no column names provided!";
+		# check and open file
+		my $filename = $self->check_file($args{in});
+		unless ($filename) {
+			carp sprintf "file '%s' does not exist!", $args{in};
 			return;
 		}
-		my $feature = $args{features} || 'feature';
-		$data = generate_data_structure($feature, @datasets);
+		$self->add_file_metadata($filename);
+		my $fh = $self->open_to_read_fh or return;
+		$self->{fh} = $fh;
+		$self->{mode} = 0; # read mode
+		
+		# parse column headers
+		$self->{header_line_count} = 0;
+		$self->parse_headers;
+		$self->{data_table}->[0] = $self->{'column_names'}; 
+		$self->{line_count} = $self->{header_line_count};
+		
+		# look for potential start columns to convert from 0-based
+		$self->{'0based_starts'} = [];
+		if ($self->bed or $self->{'ucsc'}) {
+			foreach my $name (qw(start txStart cdsStart peak)) {
+				my $c = $self->find_column($name);
+				next unless defined $c;
+				next if $self->metadata($c, 'base');
+				push @{ $self->{'0based_starts'} }, $c;
+			}
+		}
+		
+		# potential strand column that may need to be converted
+		$self->{plusminus_count} = 0;
+	}
+	
+	# prepare to write to a new stream
+	elsif ($args{out}) {
 		
 		# add file name information
-		my ($basename, $path, $extension) = parse_filename($args{filename});
-		$data->{filename}  = $args{filename};
-		$data->{basename}  = $basename;
-		$data->{path}      = $path;
-		$data->{extension} = $extension;
-		
-		# add extra information for checking strand and start information
-		$data->{strand_check} = undef;
-		$data->{column_starts} = [];
+		$self->add_file_metadata($args{out});
 		
 		# we will not open the file handle quite yet in case the user 
 		# wants to modify metadata
-		$mode = 1; # write mode
+		$self->{mode} = 1; # set to write mode
+		
+		# get names of columns user may have passed
+		my @columns;
+		if (exists $args{columns}) {
+			@columns = @{ $args{columns} };
+		}
+		elsif (exists $args{datasets}) {
+			@columns = @{ $args{datasets} };
+		}
+		
+		# add the column names
+		if (@columns) {
+			foreach my $c (@columns) {
+				$self->add_column($c);
+			}
+		}
+		elsif (exists $args{gff} and $args{gff}) {
+			# use standard names for the number of columns indicated
+			# we trust that the user knows the subtle difference between gff versions
+			$self->add_gff_metadata($args{gff});
+			$self->{'data_table'}->[0] = $self->{'column_names'};
+		}
+		elsif (exists $args{bed} and $args{bed}) {
+			# use standard names for the number of columns indicated
+			unless ($args{bed} =~ /^\d{1,2}$/ and $args{bed} >= 3) {
+				carp "bed parameter must be an integer 3-12!";
+				return;
+			}	
+			$self->add_bed_metadata($args{bed});
+			$self->{'data_table'}->[0] = $self->{'column_names'};
+		}
+		elsif (exists $args{ucsc} and $args{ucsc}) {
+			# a ucsc format such as refFlat, genePred, or genePredExt
+			my $u = $self->add_ucsc_metadata($args{ucsc});
+			unless ($u) {
+				carp "unrecognized number of columns for ucsc format!";
+				return;
+			};
+			$self->{'data_table'}->[0] = $self->{'column_names'};
+		}
+		# else it will be an empty object with no columns
+		
+		# add feature
+		$args{feature} ||= $args{features} || undef;
+		$self->feature($args{feature}) unless $self->feature;
+		
+		# look for potential start columns to convert from 0-based
+		$self->{'0based_starts'} = [];
+		if ($self->{'ucsc'} or $self->{'bed'}) {
+			my @starts;
+			foreach my $name (qw(start txStart cdsStart peak)) {
+				my $c = $self->find_column($name);
+				next unless defined $c;
+				next if $self->metadata($c, 'base');
+				push @{ $self->{'0based_starts'} }, $c;
+			}
+		}
 	}
 	
-	$data->{fh} = $fh;
-	$data->{mode} = $mode;
-	return bless $data, $class;
+	return $self;
 }
 
 
 sub duplicate {
-	my $self = shift;
-	my $filename = shift;
+	my ($self, $filename) = @_;
 	unless ($filename) {
 		carp "a new filename must be provided!";
 		return;
@@ -573,50 +629,47 @@ sub duplicate {
 	}
 	
 	# duplicate the data structure
-	my $data = generate_data_structure( $self->feature, $self->list_columns );
+	my $columns = $self->list_columns;
+	my $Dup = $self->new(
+		'out' => $filename, 
+		'columns' => $columns,
+	) or return;
 	
 	# copy the metadata
 	for (my $i = 0; $i < $self->number_columns; $i++) {
 		# column metadata
 		my %md = $self->metadata($i);
-		$data->{$i} = \%md;
+		$Dup->{$i} = \%md;
 	}
-	foreach (qw(program db bed gff ucsc headers)) {
+	foreach (qw(feature program db bed gff ucsc headers)) {
 		# various keys
-		$data->{$_} = $self->{$_};
+		$Dup->{$_} = $self->{$_};
 	}
-	$data->{column_starts} = [ @{ $self->{column_starts} } ];
+	$Dup->{'0based_starts'} = [ @{ $self->{'0based_starts'} } ];
 	my @comments = $self->comments;
-	push @{$data->{other}}, @comments;
+	push @{$Dup->{comments}}, @comments;
 	
-	# add file name information
-	undef $data->{fh};
-	$data->{mode} = 1;
-	my ($basename, $path, $extension) = parse_filename($filename);
-	$data->{filename}  = $filename;
-	$data->{basename}  = $basename;
-	$data->{path}      = $path;
-	$data->{extension} = $extension;
-	
-	return bless $data, 'Bio::ToolBox::Data::Stream';
+	return $Dup;
 }
 
 
 
 ### Column manipulation
 
-# see also Bio::ToolBox::Data::common for imported methods
-
 sub add_column {
 	my ($self, $name) = @_;
 	return unless $name;
+	unless ($self->mode) {
+		cluck "We have a read-only Stream object, cannot add columns";
+		return;
+	}
 	if (defined $self->{fh}) {
 		# Stream file handle is opened
 		cluck "Cannot modify columns when a Stream file handle is opened!";
 		return;
 	}
 	
-	my $column = $self->{number_columns};
+	my $column = $self->number_columns;
 	$self->{$column} = {
 		'name'      => $name,
 		'index'     => $column,
@@ -629,6 +682,10 @@ sub add_column {
 
 sub copy_column {
 	my $self = shift;
+	unless ($self->mode) {
+		cluck "We have a read-only Stream object, cannot add columns";
+		return;
+	}
 	if (defined $self->{fh}) {
 		# Stream file handle is opened
 		cluck "Cannot modify columns when a Stream file handle is opened!";
@@ -653,25 +710,22 @@ sub next_row {
 		return;
 	}
 	
-	# process the data line, converting strand and 0-based coordinates as necessary
+	# read and add the next line in the file
 	my $line = $self->{fh}->getline;
 	return unless $line;
-	my ($linedata, $plusminus) = process_data_line($line, $self->number_columns, 
-		$self->strand_column, @{ $self->{column_starts} });
+	$self->{line_count}++;
+	if ($line =~ /^#/) {
+		$self->add_comment($line);
+		return $self->next_row;
+	}
+	return $self->next_row if $line !~ m/\w+/;
 	
-	# update the data table
-	# it will always be row 1
-	$self->{data_table}->[1] = $linedata;
-	
-	# update metadata as necessary
-	if (defined $self->{strand_check} and $plusminus) {
-		$self->metadata($self->strand_column, 'strand_style', 'plusminus');
-		my $auto = $self->metadata($self->strand_column, 'AUTO');
-		if ($auto) {
-			# update automatically generated metadata
-			$self->metadata($self->strand_column, 'AUTO', $auto++);
-		}
-		undef $self->{strand_check}; # we no longer need to check this
+	# add the current line to the data table as row 1
+	splice( @{ $self->{data_table} }, 1, 1); # remove the old line
+	my $added = $self->add_data_line($line);
+	unless ($added) {
+		cluck "could not process line '$line'!";
+		return $self->next_row;
 	}
 	
 	# return the feature
@@ -683,7 +737,7 @@ sub next_row {
 
 
 sub add_row {
-	shift->write_row(@_);
+	return shift->write_row(@_);
 }
 
 
@@ -698,20 +752,19 @@ sub write_row {
 	# open the file handle if it hasn't been opened yet
 	unless (defined $self->{fh}) {
 		# we first write a standard empty data file with metadata and headers
-		my $filename = write_data_file(
-			data     => $self,
-			filename => $self->filename,
-		);
-		unless ($filename) {
+		my $newfile = $self->write_file($self->filename);
+		unless ($newfile) {
 			die "unable to write file!";
 		}
 		
 		# just in case the filename is changed when writing the file
-		$self->filename($filename);
+		if ($newfile ne $self->filename) {
+			$self->add_file_metadata($newfile);
+		}
 		
 		# then we re-open the file for appending
-		my $fh = open_to_write_fh($filename, undef, 1) or 
-			die "unable to append to file $filename!";
+		my $fh = $self->open_to_write_fh($newfile, undef, 1) or 
+			die "unable to append to file $newfile!";
 		$self->{fh} = $fh;
 	}
 	
@@ -720,80 +773,30 @@ sub write_row {
 	my @values;
 	if ($data_ref eq 'Bio::ToolBox::Data::Feature') {
 		# user passed a Feature object
-		
-		# get the values
 		@values = $data->row_values;
-		
-		# check strand information
-		unless (defined $self->{strand_check}) {
-			# we do this check only once, presuming that if it's true for the first 
-			# one, it's probably true for all
-			# too expensive to do every time we perform a write, right?
-			# must dig back into the original Stream object, which may or may not be self
-			if ($data->{data}->strand_column) {
-				if ($data->{data}->metadata( $data->{data}->strand_column, 'strand_style') 
-					eq 'plusminus'
-				) {
-					# we do need to convert strand back
-					$self->{strand_check} = 1;
-				}
-				else {
-					# we should not convert strand back
-					$self->{strand_check} = 0;
-				}
-			}
-			
-			# also check start coordinates while we're at it
-			if (scalar @{ $data->{data}->{column_starts} } > 0) {
-				$self->{column_starts} = [ @{ $data->{data}->{column_starts} } ];
-			}
-			else {
-				$self->{column_starts} = [];
-			}
-		}
 	}
 	elsif ($data_ref eq 'ARRAY') {
 		# user passed an array of values
 		@values = @$data;
-		
-		# check strand information
-		unless (defined $self->{strand_check}) {
-			# we do this check only once, presuming that if it's true for the first 
-			# one, it's probably true for all
-			# too expensive to do every time we perform a write, right?
-			# use the metadata from the current Stream object which may not be accurate
-			if ($self->strand_column) {
-				if (
-					$self->metadata($self->strand_column, 'strand_style') eq 'plusminus'
-				) {
-					# we need to convert strand back
-					$self->{strand_check} = 1;
-				}
-				else {
-					# we do not need to convert strand back
-					$self->{strand_check} = 0;
-				}
-			}
-		}
 	}
 	
 	# write the values as necessary
 	if (@values) {
 		# we have an array of values to write
 		# make adjustments as necessary
-		if ($self->{strand_check}) {
+		my $strand_i = $self->strand_column;
+		if (defined $strand_i and $values[$strand_i] =~ /\d/) {
 			# strand adjustment
-			my $i = $data->{data}->strand_column;
-			if ($values[$i] >= 0) {
-				$values[$i] = '+';
+			if ($values[$strand_i] >= 0) {
+				$values[$strand_i] = '+';
 			}
 			else {
-				$values[$i] = '-';
+				$values[$strand_i] = '-';
 			}
 		}
-		if ( @{$self->{column_starts}} ) {
+		if ( @{$self->{'0based_starts'}} ) {
 			# 0-based start coordinate adjustment
-			foreach my $c ( @{$self->{column_starts}} ) {
+			foreach my $c ( @{$self->{'0based_starts'}} ) {
 				$values[$c] -= 1;
 			}
 		}
@@ -812,6 +815,7 @@ sub write_row {
 		}
 		$self->{fh}->print($data);
 	}
+	return 1;
 }
 
 
@@ -833,6 +837,11 @@ sub close_fh {
 	$self->{fh}->close;
 }
 
+sub DESTROY {
+	my $self = shift;
+	$self->close_fh;
+}
+
 
 ####################################################
 
