@@ -5,17 +5,15 @@
 use strict;
 use Getopt::Long;
 use Pod::Usage;
-use Bio::ToolBox::data_helper qw(
-	find_column_index
-);
-use Bio::ToolBox::file_helper qw(
+use Bio::ToolBox::legacy_helper qw(
 	open_data_file
 	write_data_file
 	open_to_write_fh
-	convert_genome_data_2_gff_data
+	find_column_index
 );
+use Bio::ToolBox::Extra qw(convert_genome_data_2_gff_data);
 use Bio::ToolBox::utility;
-my $VERSION =  1.24;
+my $VERSION =  '1.30';
 
 print "\n This script will convert a data file to a GFF\n\n";
 
