@@ -27,7 +27,7 @@ use constant DATASET_HASH_LIMIT => 5001;
 		# region, and a hash returned with potentially a score for each basepair. 
 		# This may become unwieldy for very large regions, which may be better 
 		# served by separate database queries for each window.
-my $VERSION = 1.25;
+my $VERSION = '1.30';
 
 print "\n A script to collect windowed data flanking a relative position of a feature\n\n";
   
@@ -160,6 +160,7 @@ else {
 		feature => $feature,
 	) or die " unable to generate new feature list\n";
 }
+$Data->program("$0, v $VERSION");
 
 # the number of columns already in the data array
 my $startcolumn = $Data->number_columns; 
@@ -1103,8 +1104,8 @@ genomic region of the feature. Accepted values include:
 
 Optionally specify the type of data value to collect from the dataset or 
 data file. Four values are accepted: score, count, pcount, or length. 
-The default value type is score. Note that some data sources only support certain 
-types of data values. The types are detailed below.
+The default value type is score. Note that some data sources only support 
+certain types of data values. The types are detailed below.
 
 =over 4
 
@@ -1151,13 +1152,14 @@ regions (e.g. BED files). Default is false.
 
 =item --avoid
 
-Indicate whether features of the same type should be avoided when 
-calculating values in a window. Each window is checked for 
+Indicate whether search features of the same type should be avoided 
+when calculating values in a window. Each window is checked for 
 overlapping features of the same type; if the window does overlap 
 another feature of the same type, no value is reported for the 
 window. This option requires using named database features and must 
-include a feature GFF type column. The default is false (return all 
-values regardless of overlap).
+include a feature GFF type column. This is useful to avoid scoring 
+windows that overlap a neighboring gene, for example. The default is 
+false (return all values regardless of overlap).
 
 =item --long
 
@@ -1167,8 +1169,10 @@ data (microarray data or sequence coverage). Normally long features are
 only recorded at their midpoint, leading to inaccurate representation at 
 some windows. This option forces the program to collect data separately 
 at each window, rather than once for each file feature or region and 
-subsequently assigning scores to windows. Execution times may be 
-longer than otherwise. Default is false.
+subsequently assigning scores to windows. This may result in counting 
+features more than once if it overlaps more than one window, a result 
+that may or may not be desired. Execution time will likely increase. 
+Default is false.
 
 =item --log
 
